const ExtractTextPlugin = require( 'extract-text-webpack-plugin' );
const ManifestPlugin = require( 'webpack-manifest-plugin' );
const debug = process.env.NODE_ENV !== 'production';
const glob = require( 'glob' );
const path = require( 'path' );
const autoprefixer = require( 'autoprefixer' );
const spawn = require( 'child_process' ).spawnSync;

// Main CSS loader for everything but blocks..
const mainCSSExtractTextPlugin = new ExtractTextPlugin( {
	filename: function( getPath ) {
		return getPath( debug ? '[name].css' : '[name].min.css' ).replace( '/js/', '/css/' );
	},
} );

// Configuration for the ExtractTextPlugin.
const extractConfig = {
	use: [
		{ loader: 'raw-loader' },
		{
			loader: 'postcss-loader',
			options: {
				plugins: [
					autoprefixer,
				],
			},
		},
		{
			loader: 'sass-loader',
			options: {
				outputStyle: debug ? 'nested' : 'compressed',
				sourceMap: debug ? 'inline' : false,
				includePaths: [
					'packages/style-guide/src',
				],
			},
		},
	],
};

/*
Convert the wildcard entry points into an entry object suitable for Webpack consumption.

This requires an object where the key is the path to the destination file without a file extension
and the value is the path to the source file.

For Example:
[ 'pro/dashboard/entries/dashboard.js' ]
{ 'pro/dashboard/js/dashboard': './pro/dashboard/entries/dashboard.js' }
*/
const entries = glob.sync( 'pro/**/entries/*.js' ).reduce( function( acc, entry ) {
	const baseName = path.basename( entry, '.js' );
	const out = path.join( entry, '..', '..', 'js', baseName );

	acc[ out ] = './' + entry; // The entry needs to be marked as relative to the current directory.

	return acc;
}, {} );

function camelCaseDash( string ) {
	return string.replace(
		/-([a-z])/g,
		( match, letter ) => letter.toUpperCase()
	);
}
const formatRequest = ( request ) => {
	// '@wordpress/api-fetch' -> [ '@wordpress', 'api-fetch' ]
	const [ , name ] = request.split( '/' );

	// { this: [ 'wp', 'apiFetch' ] }
	return {
		this: [ 'wp', camelCaseDash( name ) ],
	};
};

const wordpressExternals = ( context, request, callback ) => {
	if ( /^@wordpress\//.test( request ) ) {
		callback( null, formatRequest( request ), 'this' );
	} else {
		callback();
	}
};

const externals = [
	{
		react: 'React',
		'react-dom': 'ReactDOM',
		tinymce: 'tinymce',
		moment: 'moment',
		jquery: 'jQuery',
		lodash: 'lodash',
		'lodash-es': 'lodash',
	},
	wordpressExternals,
];

const config = {
	context: __dirname,
	devtool: debug ? 'inline-sourcemap' : false,
	mode: debug ? 'development' : 'production',
	entry: entries,
	output: {
		path: __dirname,
		filename: debug ? '[name].js' : '[name].min.js',
		jsonpFunction: 'itsecWebpackJsonP',
	},
	externals,
	module: {
		rules: [
			{ parser: { amd: false } },
			{
				test: /\.js$/,
				exclude: /node_modules/,
				use: [
					{
						loader: 'babel-loader',
					},
				],
			},
			{
				test: /\.s?css$/,
				use: mainCSSExtractTextPlugin.extract( extractConfig ),
			},
			{
				test: /\.svg$/,
				exclude: /node_modules/,
				use: [
					{
						loader: 'svg-react-loader',
						query: {
							classIdPrefix: 'itsec-icon-[name]-[hash:5]__',
						},
					},
				],
			},
		],
	},
	plugins: [
		mainCSSExtractTextPlugin,
	],
	resolve: {
		modules: [
			path.resolve( './' ),
			path.resolve( './node_modules' ),
		],
	},
	optimization: {

	},
};

if ( ! debug ) {
	config.optimization.splitChunks = {
		chunks: 'initial',
		cacheGroups: {
			recharts: {
				test: /[\\/]node_modules[\\/](recharts)[\\/]/,
				name: 'vendor/recharts',
				enforce: true,
			},
			'react-select': {
				test: /[\\/]node_modules[\\/](react-select)[\\/]/,
				name: 'vendor/react-select',
				enforce: true,
			},
		},
	};

	config.plugins.push( new ManifestPlugin( {
		fileName: 'manifest.php',
		generate( seed, files ) {
			const manifest = {};
			const split = [];

			for ( const file of files ) {
				if ( ! file.chunk ) {
					continue;
				}

				const fileManifest = {
					hash: file.chunk.hash,
					runtime: file.chunk.hasRuntime(),
					dependencies: [],
					vendors: [],
				};

				if ( ! file.chunk.hasRuntime() ) {
					split.push( file );
				}

				file.chunk.getModules().forEach( ( module ) => {
					if ( module.external && module.userRequest ) {
						if ( module.userRequest.includes( '@wordpress/' ) ) {
							fileManifest.dependencies.push( `wp-${ module.userRequest.replace( '@wordpress/', '' ) }` );
						} else {
							fileManifest.dependencies.push( module.userRequest );
						}
					}
				} );

				fileManifest.dependencies.sort();

				manifest[ file.chunk.name ] = fileManifest;
			}

			for ( const file of split ) {
				file.chunk.groupsIterable.forEach( ( group ) => {
					manifest[ group.name ].vendors.push( file.chunk.name );
				} );
			}

			return manifest;
		},
		serialize( data ) {
			const out = spawn( 'php', [
				path.join( __dirname, 'bin', 'json-to-php.php' ),
				JSON.stringify( data ),
			] );

			if ( out.status !== 0 ) {
				throw Error( 'Failed to generate PHP manifest.' );
			}

			return `<?php return ${ out.stdout };`;
		},
	} ) );
}

module.exports = config;
